'use strict';

// //////////////////////////////////////////////////////////////////////////////
// DISCLAIMER
//
// Copyright 2020 ArangoDB GmbH, Cologne, Germany
//
// The Programs (which include both the software and documentation) contain
// proprietary information of ArangoDB GmbH; they are provided under a license
// agreement containing restrictions on use and disclosure and are also
// protected by copyright, patent and other intellectual and industrial
// property laws. Reverse engineering, disassembly or decompilation of the
// Programs, except to the extent required to obtain interoperability with
// other independently created software or as specified by law, is prohibited.
//
// It shall be the licensee's responsibility to take all appropriate fail-safe,
// backup, redundancy, and other measures to ensure the safe use of
// applications if the Programs are used for purposes such as nuclear,
// aviation, mass transit, medical, or other inherently dangerous applications,
// and ArangoDB GmbH disclaims liability for any damages caused by such use of
// the Programs.
//
// This software is the confidential and proprietary information of ArangoDB
// GmbH. You shall not disclose such confidential and proprietary information
// and shall use it only in accordance with the terms of the license agreement
// you entered into with ArangoDB GmbH.
//
// @author Heiko Kernbach
// //////////////////////////////////////////////////////////////////////////////

const internal = require('internal');
const arangosh = require('@arangodb/arangosh');
const satelliteGraphCommon = require('@arangodb/satellite-graph-common');
const generalGraphClient = require('@arangodb/general-graph');
const _ = require('lodash');

const db = internal.db;
const SatelliteGraph = satelliteGraphCommon.__SatelliteGraphClass;
const Graph = satelliteGraphCommon.__GraphClass;
const GeneralGraphClient = generalGraphClient.__GraphClass;

const GRAPH_PREFIX = '_api/gharial/';

exports._graph = function (graphName) {
  const uri = GRAPH_PREFIX + encodeURIComponent(graphName);
  const requestResult = arangosh.checkRequestResult(db._connection.GET(uri));
  const g = requestResult.graph;
  if (g.isSatellite) {
    return new SatelliteGraph(g);
  }
  return new Graph(g);
};

exports._create = function (name, edgeDefinitions, orphans) {
  const data = {
    options: {
      replicationFactor: "satellite"
    }
  };
  if (name) {
    data.name = name;
  }
  if (edgeDefinitions) {
    data.edgeDefinitions = edgeDefinitions;
  }
  if (orphans) {
    data.orphanCollections = orphans;
  }

  const uri = GRAPH_PREFIX;
  const requestResult = arangosh.checkRequestResult(db._connection.POST(uri, JSON.stringify(data)));
  db._flushCache();
  // should always be a SatelliteGraph
  return new SatelliteGraph(requestResult.graph);
};

exports._drop = function (graphName, dropCollections) {
  let uri = GRAPH_PREFIX + encodeURIComponent(graphName);
  if (dropCollections) {
    uri += "?dropCollections=true";
  }
  const requestResult = arangosh.checkRequestResult(db._connection.DELETE(uri));
  if (dropCollections) {
    db._flushCache();
  }
  return requestResult.result;
};

exports._list = function () {
  const uri = GRAPH_PREFIX;
  const requestResult = arangosh.checkRequestResult(db._connection.GET(uri));
  const graphs = requestResult.graphs;

  const result = [];
  _.each(graphs, function (graph) {
    result.push(graph._key);
  });
  return result;
};

exports._listObjects = function () {
  const uri = GRAPH_PREFIX;
  const requestResult = arangosh.checkRequestResult(db._connection.GET(uri));
  return requestResult.graphs;
};

exports._exists = satelliteGraphCommon._exists;

exports._relation = satelliteGraphCommon._relation;
exports._edgeDefinitions = satelliteGraphCommon._edgeDefinitions;
exports._extendEdgeDefinitions = satelliteGraphCommon._extendEdgeDefinitions;
exports._registerCompatibilityFunctions = satelliteGraphCommon._registerCompatibilityFunctions;

SatelliteGraph.prototype.__updateDefinitions = GeneralGraphClient.prototype.__updateDefinitions;
SatelliteGraph.prototype._extendEdgeDefinitions = GeneralGraphClient.prototype._extendEdgeDefinitions;
SatelliteGraph.prototype._editEdgeDefinitions = GeneralGraphClient.prototype._editEdgeDefinitions;
SatelliteGraph.prototype._addVertexCollection = GeneralGraphClient.prototype._addVertexCollection;
SatelliteGraph.prototype._removeVertexCollection = GeneralGraphClient.prototype._removeVertexCollection;
SatelliteGraph.prototype._deleteEdgeDefinition = GeneralGraphClient.prototype._deleteEdgeDefinition;
